// 
// PROJECT NAME : Boca Systems Sample C# Project - Bi-directional Communication
// AUTHOR       : michael@bocasystems.com
//
// This example source code is intended for Boca Systems Customers interested in bi-directional communication between
// a Windows PC (with .Net Framework 4.0 installed) and a Boca printer local to the PC or on a local network with a shared
// printer driver.  Communication is performed at the interface level WITHOUT the use of printer drivers or at the driver 
// level with the use of Boca Systems printer drivers.  Raw text FGL commands are demonstrated in both approaches.  
//
// Bi-directional Communication without printer drivers via Serial, Parallel, USB in HID mode and Ethernet (including Wifi) 
// is demonstrated in the following files.
// 1. Serial - COMSupport.cs
// 2. Parallel - LPTSupport.cs
// 3. HID - USBHIDSupport.cs
// 4. Ethernet - IPSupport.cs
// 
// Bi-directional Communication with the Boca Systems printer drivers installed regardless of port selection 
// including local network shared drivers, is demonstrated in DriverSupport.cs.  If you need the Boca Systems Windows printer
// drivers they can be downloaded from our web site or look in the parent directory for this project and you will find
// printer_drivers.exe.  Run this self-extracting zip file to install the drivers onto the PC so the drivers are ready
// and available when the printer is plugged in and installed.
//
// Most Boca Systems with a USB port are shipped in "Printer" mode (class 7 device )and will install in a plug and play fashion.
// In this mode Windows automatically installs "USB Printing Support" for low level USB communication with a printer driver 
// on top of that.  The USB port can be also configured into "HID" mode (class 3 device) and installed as an Human Interface Device
// using Windows HID drivers for low level USB communication with no printer driver like any other interface (lpt, com, etc).
// If you have already installed the USB printer as a "printer" and now want to change over to HID mode, it is important to uninstall
// the printer driver and "USB Printing Support" BEFORE you reconfigure the USB port as an HID interface.  The USB Device Type
// configuration settings can be altered from the factory menu of the printers equipped with a display or by 
// sending configuration commands to the printer.
//
// These items can be reconfigured in an FGL printer, by sending FGL text commands (using this tool) to the printer.
//
// FGL     
// COMMAND  DESCRIPTION
// =======  ================================
// <usbp> - USB Device Type -> Printer Mode
// <usbh> - USB Device Type -> HID Mode
//
// When a Boca Systems printer is built and shipped all ports in the printer are setup with bi-directional capabilities
// enabled, except the parallel port.  The parallel port is initially setup in uni-directional mode (write only).  If 
// you intend to use the parallel port for printing and reading status back from the printer, then the bi-directional
// capabilities of the parallel port must be enabled before attempting to use the "Read Status" button or the 
// "wait for status" option.  This can be achieved from the factory menu of the printers equipped with a display or by 
// sending configuration commands to the printer.  For this, it will be necessary to perfom 2 functions:
//
// 1. Enable Bi-Directional parallel port, first and then,
// 2. Include parallel port on return status list.
//
// These items can be reconfigured in an FGL printer, by sending FGL text commands (using this tool) to the printer.
//
// FGL     
// COMMAND DESCRIPTION
// ======= ================================
// <bpe> - Bidi Parallel enabled
// <se7> - Status return on USB/SER/PAR
//
// Each support file provides the same following four basic port functions: Open, Close, Write and Read.  By using these
// functions, one can in theory develop a ticketing system around them.
// 
// Main routines of interest:
// 1. port_open()       opens whichever port is selected
// 2. port_shutdown()   close which port is open
// 3. transmit()        transmit strings out the open port
// 4. transmitb()       transmit byte array out the open port
// 5. receive()         read incoming data from the printer in seperate thread, use callback function to display status
//
// The buttons on the main screen FrmBoca will lead to code demonstrating common functions.
//  1. FGL Test Ticket - How to use FGL commands to lay out an FGL ticket.  These commands could be stored in a text file vs hardcoded
//     in the standard() routine.
//  2. PCL Test Ticket - How to use a prn file (output from a driver) to print an HP/PCL emulation ticket.
//  3. FGL Form Feed - Demonstrate use of FGL formfeed command <p>
//  4. PCL Form Feed - Demonstrate use of ASCII value 12 which is the standard HP emulation form feed
//  5. Send Text - Demonstrate FGL text commands such as:  <RC100,100><F11>This is a test.<p>
//  6. Send ASCII value - Demonstrate sending ASCII value such as form feed above and Escape (Ascii 27 = 0x1B)
//  7. Graphics - Demonstrate how to print a graphic file or download a graphic file (logo) to memory for printing later
//  8. TTF - How to download TTF files into the printer memory for use later when printing 
//  9. Other - How to download other files such as soft font files or text files filled with FGL commands.
// 10. Clear Download Space - How to clear the user memory download space of previously downloaded soft font, ttf, logos etc.
// 11. Set Path - For Boca printers with multiple paths.
// 12. Sound - After reading return status from the printer the application will play a wav file to inform the user when the
//     printer is out of stock, jammed etc.  See establish_status() routine.
// 13. Download logo test - demonstrates how to download monochrome pcx & bmp graphics file into Flash memory.  Also shows
//     how to assign specific file ID numbers and use them later to print the logo from flash onto the ticket.
//
// Technical notes:
// Overlapped I/O     - In this, my first C# project, I have been unable to get overlapped IO to execute properly on the USB/HID interface.  
//                      The previous version of the customer interface program I wrote years ago in VB6 has overlapped IO executing
//                      correctly.  Instead of holding up the whole project, I decided to release what I have.  I am currently performing
//                      a polling read function in a seperate thread.  When data is returned from the printer a callback function (delegate)
//                      is used to send the data back to the main thread to establish_status().  When I put the Overlapped IO pieces in place
//                      the code runs, I receive no error, but the READ function also returns no data.  I will continue to work on this issue.
//                      Should I solve it I will release an updated version.  If someone figures out this (or any) problem, please share the 
//                      solution with me.  Since HID IO is still blocking, at shutdown I send a final <S1> to FGL printer so read will exit
//                      when the Xon is returned from the printer.
//
// Callback function  - delegate used to transmit data back to the main thread
//
// Threading          - used to be like spawning a seperate application, but the threads maintain communication
//
// m_parent           - used by child app/thread to refer back to the parent app/main thread.
//
// form close event   - tried to find something like the form_unload in VB.  Ended up using Dispose() in FrmBoca.Designer.cs 
//                      to call port_shutdown.  From here the checking_for_data boolean is set to false to allow read threads to exit
//                      "while loop" once the <S1> is sent.
//
// .Net Framework 4.0 - Windows API calls in this code rely on .Net Framework 4.0.  To update .Net Framework go to the following
//                      MicroSoft web site http://www.microsoft.com/download/en/details.aspx?id=17851 and download version 4.
//
// On the FrmBoca main screen I added 3 links
// 1. www.bocasystems.com main web page
// 2. Download FGL Programmers guide fgl46.doc
// 3. Boca Systems online technical support can be used to descibe a problem or ask a question.  It will be emailed directly to
//    the technical support group and an email response will be forthcoming.
//
using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Threading;
using System.IO;

namespace boca_sample
{

    static class Program
    {

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            Application.Run(new FrmBoca());
        } 

    }           //end of class Program
}               //end of namespace